@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import datetime
+import json
 import os
 
 from sphinx.ext import autodoc
@@ -21,6 +22,8 @@ copyright = (u'2014-{}, Markus Unterwaditzer & contributors'
 release = vdirsyncer.__version__
 version = '.'.join(release.split('.')[:2])  # The short X.Y version.
 
+rst_epilog = '.. |vdirsyncer_version| replace:: %s' % release
+
 exclude_patterns = ['_build']
 
 pygments_style = 'sphinx'
@@ -84,6 +87,27 @@ def github_issue_role(name, rawtext, text, lineno, inliner,
     return [node], []
 
 
+def format_storage_config(cls, header=True):
+    if header is True:
+        yield '[storage example_for_{}]'.format(cls.storage_name)
+    yield 'type = "{}"'.format(cls.storage_name)
+
+    from vdirsyncer.storage.base import Storage
+    from vdirsyncer.utils import get_storage_init_specs
+    handled = set()
+    for spec in get_storage_init_specs(cls, stop_at=Storage):
+        defaults = spec.defaults or ()
+        defaults = dict(zip(spec.args[-len(defaults):], defaults))
+        for key in spec.args[1:]:
+            if key in handled:
+                continue
+            handled.add(key)
+
+            comment = '' if key not in defaults else '#'
+            value = defaults.get(key, '...')
+            yield '{}{} = {}'.format(comment, key, json.dumps(value))
+
+
 class StorageDocumenter(autodoc.ClassDocumenter):
     '''Custom formatter for auto-documenting storage classes. It assumes that
     the first line of the class' docstring is its own paragraph.
@@ -111,7 +135,6 @@ class StorageDocumenter(autodoc.ClassDocumenter):
                       '<autodoc>')
 
     def get_doc(self, encoding=None, ignore=1):
-        from vdirsyncer.cli.utils import format_storage_config
         rv = autodoc.ClassDocumenter.get_doc(self, encoding, ignore)
         config = [u'    ' + x for x in format_storage_config(self.object)]
         rv[0] = rv[0][:1] + [u'::', u''] + config + [u''] + rv[0][1:]
