@@ -26,10 +26,12 @@ except ImportError:
 else:
     del pytest_benchmark
 
+
+settings.suppress_health_check = [HealthCheck.too_slow]
+
 settings.register_profile("ci", settings(
     max_examples=1000,
     verbosity=Verbosity.verbose,
-    suppress_health_check=[HealthCheck.too_slow]
 ))
 settings.register_profile("deterministic", settings(
     derandomize=True,
