@@ -1,20 +1,16 @@
 import json
 import os
 import string
+from configparser import RawConfigParser
 from itertools import chain
 
 from click_threading import get_ui_worker
 
-from . import cli_logger
 from .fetchparams import expand_fetch_params
 from .utils import storage_class_from_config
 from .. import PROJECT_HOME, exceptions
 from ..utils import cached_property, expand_path
 
-try:
-    from ConfigParser import RawConfigParser
-except ImportError:
-    from configparser import RawConfigParser
 
 GENERAL_ALL = frozenset(['status_path'])
 GENERAL_REQUIRED = frozenset(['status_path'])
@@ -146,47 +142,10 @@ class _ConfigReader:
         return self._general, self._pairs, self._storages
 
 
-def _parse_config_value(value):
-    try:
-        return json.loads(value)
-    except ValueError:
-        pass
-
-    for wrong, right in [
-        (('on', 'yes'), 'true'),
-        (('off', 'no'), 'false'),
-        (('none',), 'null')
-    ]:
-        if value.lower() in wrong + (right,):
-            raise ValueError('You probably meant {} instead of {}. Surround '
-                             'your string with double-quotes if not.'
-                             .format(right, value))
-
-    if '#' in value:
-        raise ValueError('Invalid value:{}\n'
-                         'Use double quotes (") if you want to use hashes in '
-                         'your value.')
-
-    if len(value.splitlines()) > 1:
-        # ConfigParser's barrier for mistaking an arbitrary line for the
-        # continuation of a value is awfully low. The following example will
-        # also contain the second line in the value:
-        #
-        # foo = bar
-        #  # my comment
-        raise ValueError('No multiline-values allowed:\n{}'.format(value))
-
-    cli_logger.warning('Soon, all strings have to be in double quotes. Please '
-                       'replace {} with {}'
-                       .format(value, json.dumps(value)))
-
-    return value
-
-
 def _parse_options(items, section=None):
     for key, value in items:
         try:
-            yield key, _parse_config_value(value)
+            yield key, json.loads(value)
         except ValueError as e:
             raise ValueError('Section "{}", option "{}": {}'
                              .format(section, key, e))
@@ -314,6 +273,9 @@ class PairConfig(object):
     @cached_property
     def partial_sync(self):
         partial_sync = self._partial_sync
+        # We need to use UserError here because ValueError is not
+        # caught at the time this is expanded.
+
         if partial_sync is not None:
             cls_a, _ = storage_class_from_config(self.config_a)
             cls_b, _ = storage_class_from_config(self.config_b)
@@ -322,14 +284,17 @@ class PairConfig(object):
                not self.config_a.get('read_only', False) and \
                not cls_b.read_only and \
                not self.config_b.get('read_only', False):
-                raise ValueError('`partial_sync` is only effective if one '
-                                 'storage is read-only.')
+                raise exceptions.UserError(
+                    '`partial_sync` is only effective if one storage is '
+                    'read-only. Use `read_only = true` in exactly one storage '
+                    'section.'
+                )
 
         if partial_sync is None:
             partial_sync = 'revert'
 
         if partial_sync not in ('ignore', 'revert', 'error'):
-            raise ValueError('Invalid value for `partial_sync`.')
+            raise exceptions.UserError('Invalid value for `partial_sync`.')
 
         return partial_sync
 
@@ -347,12 +312,15 @@ class CollectionConfig(object):
 load_config = Config.from_filename_or_environment
 
 
-def _resolve_conflict_via_command(a, b, command, a_name, b_name):
+def _resolve_conflict_via_command(a, b, command, a_name, b_name,
+                                  _check_call=None):
     import tempfile
     import shutil
-    import subprocess
 
-    from ..utils.vobject import Item
+    if _check_call is None:
+        from subprocess import check_call as _check_call
+
+    from ..vobject import Item
 
     dir = tempfile.mkdtemp(prefix='vdirsyncer-conflict.')
     try:
@@ -364,7 +332,8 @@ def _resolve_conflict_via_command(a, b, command, a_name, b_name):
         with open(b_tmp, 'w') as f:
             f.write(b.raw)
 
-        subprocess.check_call(command + [a_tmp, b_tmp])
+        command[0] = expand_path(command[0])
+        _check_call(command + [a_tmp, b_tmp])
 
         with open(a_tmp) as f:
             new_a = f.read()
