@@ -2,11 +2,11 @@
 
 import contextlib
 import errno
-import hashlib
 import importlib
 import itertools
 import json
 import os
+import queue
 import sys
 
 from atomicwrites import atomic_write
@@ -17,22 +17,14 @@ import click_threading
 
 from . import cli_logger
 from .. import BUGTRACKER_HOME, DOCS_HOME, exceptions
-from ..sync import IdentConflict, PartialSync, StorageEmpty, SyncConflict
+from ..sync import IdentConflict, PartialSync, StorageEmpty, SyncConflict, \
+    SqliteStatus
 from ..utils import expand_path, get_storage_init_args
 
-try:
-    import Queue as queue
-except ImportError:
-    import queue
-
 
 STATUS_PERMISSIONS = 0o600
 STATUS_DIR_PERMISSIONS = 0o700
 
-# Increase whenever upgrade potentially breaks discovery cache and collections
-# should be re-discovered
-DISCOVERY_CACHE_VERSION = 1
-
 
 class _StorageIndex(object):
     def __init__(self):
@@ -47,7 +39,9 @@ class _StorageIndex(object):
             remotestorage_calendars=(
                 'vdirsyncer.storage.remotestorage.RemoteStorageCalendars'),
             google_calendar='vdirsyncer.storage.google.GoogleCalendarStorage',
-            google_contacts='vdirsyncer.storage.google.GoogleContactsStorage'
+            google_contacts='vdirsyncer.storage.google.GoogleContactsStorage',
+            etesync_calendars='vdirsyncer.storage.etesync.EtesyncCalendars',
+            etesync_contacts='vdirsyncer.storage.etesync.EtesyncContacts'
         )
 
     def __getitem__(self, name):
@@ -70,7 +64,6 @@ class JobFailed(RuntimeError):
     pass
 
 
-# TODO: Making this a decorator would be nice
 def handle_cli_error(status_name=None, e=None):
     '''
     Print a useful error message for the current exception.
@@ -172,223 +165,25 @@ def get_status_name(pair, collection):
     return pair + '/' + collection
 
 
-def _get_collections_cache_key(pair):
-    m = hashlib.sha256()
-    j = json.dumps([
-        DISCOVERY_CACHE_VERSION,
-        pair.collections,
-        pair.config_a,
-        pair.config_b,
-    ], sort_keys=True)
-    m.update(j.encode('utf-8'))
-    return m.hexdigest()
-
-
-def collections_for_pair(status_path, pair, from_cache=True,
-                         list_collections=False):
-    '''Determine all configured collections for a given pair. Takes care of
-    shortcut expansion and result caching.
-
-    :param status_path: The path to the status directory.
-    :param from_cache: Whether to load from cache (aborting on cache miss) or
-        discover and save to cache.
-
-    :returns: iterable of (collection, (a_args, b_args))
-    '''
-    cache_key = _get_collections_cache_key(pair)
-    if from_cache:
-        rv = load_status(status_path, pair.name, data_type='collections')
-        if rv and rv.get('cache_key', None) == cache_key:
-            return list(_expand_collections_cache(
-                rv['collections'], pair.config_a, pair.config_b
-            ))
-        elif rv:
-            raise exceptions.UserError('Detected change in config file, '
-                                       'please run `vdirsyncer discover {}`.'
-                                       .format(pair.name))
-        else:
-            raise exceptions.UserError('Please run `vdirsyncer discover {}` '
-                                       ' before synchronization.'
-                                       .format(pair.name))
-
-    cli_logger.info('Discovering collections for pair {}'
-                    .format(pair.name))
-
-    # We have to use a list here because the special None/null value would get
-    # mangled to string (because JSON objects always have string keys).
-    rv = list(_collections_for_pair_impl(status_path, pair,
-                                         list_collections=list_collections))
-
-    save_status(status_path, pair.name, data_type='collections',
-                data={
-                    'collections': list(
-                        _compress_collections_cache(rv, pair.config_a,
-                                                    pair.config_b)
-                    ),
-                    'cache_key': cache_key
-                })
-    return rv
-
-
-def _compress_collections_cache(collections, config_a, config_b):
-    def deduplicate(x, y):
-        rv = {}
-        for key, value in x.items():
-            if key not in y or y[key] != value:
-                rv[key] = value
-
-        return rv
-
-    for name, (a, b) in collections:
-        yield name, (deduplicate(a, config_a), deduplicate(b, config_b))
-
-
-def _expand_collections_cache(collections, config_a, config_b):
-    for name, (a_delta, b_delta) in collections:
-        a = dict(config_a)
-        a.update(a_delta)
-
-        b = dict(config_b)
-        b.update(b_delta)
-
-        yield name, (a, b)
-
-
-def _discover_from_config(config):
-    storage_type = config['type']
-    cls, config = storage_class_from_config(config)
-
-    discovered = []
-
-    try:
-        discovered.extend(cls.discover(**config))
-    except NotImplementedError:
-        pass
-    except Exception:
-        return handle_storage_init_error(cls, config)
-
-    rv = {}
-    for args in discovered:
-        args['type'] = storage_type
-        rv[args['collection']] = args
-    return rv
-
-
-def _handle_collection_not_found(config, collection, e=None):
-    storage_name = config.get('instance_name', None)
-
-    cli_logger.warning('{}No collection {} found for storage {}.'
-                       .format('{}\n'.format(e) if e else '',
-                               json.dumps(collection), storage_name))
-
-    if click.confirm('Should vdirsyncer attempt to create it?'):
-        storage_type = config['type']
-        cls, config = storage_class_from_config(config)
-        config['collection'] = collection
-        try:
-            args = cls.create_collection(**config)
-            args['type'] = storage_type
-            return args
-        except NotImplementedError as e:
-            cli_logger.error(e)
-
-    raise exceptions.UserError(
-        'Unable to find or create collection "{collection}" for '
-        'storage "{storage}". Please create the collection '
-        'yourself.'.format(collection=collection,
-                           storage=storage_name))
-
-
-def _print_collections(base_config, discovered):
-    instance_name = base_config['instance_name']
-    cli_logger.info('{}:'.format(instance_name))
-    for args in discovered.values():
-        collection = args['collection']
-        if collection is None:
-            continue
-
-        args['instance_name'] = instance_name
-        try:
-            storage = storage_instance_from_config(args, create=False)
-            displayname = storage.get_meta('displayname')
-        except Exception:
-            displayname = u''
-
-        cli_logger.info('  - {}{}'.format(
-            json.dumps(collection),
-            ' ("{}")'.format(displayname)
-            if displayname and displayname != collection
-            else ''
-        ))
-
-
-def _collections_for_pair_impl(status_path, pair, list_collections=False):
-    handled_collections = set()
-
-    shortcuts = pair.collections
-    if shortcuts is None:
-        shortcuts = [None]
-
-    a_discovered = _discover_from_config(pair.config_a)
-    b_discovered = _discover_from_config(pair.config_b)
-
-    if list_collections:
-        _print_collections(pair.config_a, a_discovered)
-        _print_collections(pair.config_b, b_discovered)
-
-    for shortcut in shortcuts:
-        if shortcut == 'from a':
-            collections = a_discovered
-        elif shortcut == 'from b':
-            collections = b_discovered
-        else:
-            collections = [shortcut]
-
-        for collection in collections:
-            if isinstance(collection, list):
-                collection, collection_a, collection_b = collection
-            else:
-                collection_a = collection_b = collection
-
-            if collection in handled_collections:
-                continue
-            handled_collections.add(collection)
-
-            a_args = _collection_from_discovered(a_discovered, collection_a,
-                                                 pair.config_a)
-            b_args = _collection_from_discovered(b_discovered, collection_b,
-                                                 pair.config_b)
-
-            yield collection, (a_args, b_args)
-
-
-def _collection_from_discovered(discovered, collection, config):
-    if collection is None:
-        args = dict(config)
-        args['collection'] = None
-        storage_instance_from_config(args)
-        return args
-
-    try:
-        return discovered[collection]
-    except KeyError:
-        return _handle_collection_not_found(config, collection)
-
-
-def load_status(base_path, pair, collection=None, data_type=None):
+def get_status_path(base_path, pair, collection=None, data_type=None):
     assert data_type is not None
     status_name = get_status_name(pair, collection)
     path = expand_path(os.path.join(base_path, status_name))
     if os.path.isfile(path) and data_type == 'items':
         new_path = path + '.items'
+        # XXX: Legacy migration
         cli_logger.warning('Migrating statuses: Renaming {} to {}'
                            .format(path, new_path))
         os.rename(path, new_path)
 
     path += '.' + data_type
+    return path
+
+
+def load_status(base_path, pair, collection=None, data_type=None):
+    path = get_status_path(base_path, pair, collection, data_type)
     if not os.path.exists(path):
         return None
-
     assert_permissions(path, STATUS_PERMISSIONS)
 
     with open(path) as f:
@@ -400,11 +195,7 @@ def load_status(base_path, pair, collection=None, data_type=None):
     return {}
 
 
-def save_status(base_path, pair, collection=None, data_type=None, data=None):
-    assert data_type is not None
-    assert data is not None
-    status_name = get_status_name(pair, collection)
-    path = expand_path(os.path.join(base_path, status_name)) + '.' + data_type
+def prepare_status_path(path):
     dirname = os.path.dirname(path)
 
     try:
@@ -413,6 +204,41 @@ def save_status(base_path, pair, collection=None, data_type=None, data=None):
         if e.errno != errno.EEXIST:
             raise
 
+
+@contextlib.contextmanager
+def manage_sync_status(base_path, pair_name, collection_name):
+    path = get_status_path(base_path, pair_name, collection_name, 'items')
+    status = None
+    legacy_status = None
+    try:
+        # XXX: Legacy migration
+        with open(path, 'rb') as f:
+            if f.read(1) == b'{':
+                f.seek(0)
+                # json.load doesn't work on binary files for Python 3.4/3.5
+                legacy_status = dict(json.loads(f.read().decode('utf-8')))
+    except (OSError, IOError, ValueError):
+        pass
+
+    if legacy_status is not None:
+        cli_logger.warning('Migrating legacy status to sqlite')
+        os.remove(path)
+        status = SqliteStatus(path)
+        status.load_legacy_status(legacy_status)
+    else:
+        prepare_status_path(path)
+        status = SqliteStatus(path)
+
+    yield status
+
+
+def save_status(base_path, pair, collection=None, data_type=None, data=None):
+    assert data_type is not None
+    assert data is not None
+    status_name = get_status_name(pair, collection)
+    path = expand_path(os.path.join(base_path, status_name)) + '.' + data_type
+    prepare_status_path(path)
+
     with atomic_write(path, mode='w', overwrite=True) as f:
         json.dump(data, f)
 
@@ -442,7 +268,7 @@ def storage_instance_from_config(config, create=True):
         return cls(**new_config)
     except exceptions.CollectionNotFound as e:
         if create:
-            config = _handle_collection_not_found(
+            config = handle_collection_not_found(
                 config, config.get('collection', None), e=str(e))
             return storage_instance_from_config(config, create=False)
         else:
@@ -566,30 +392,34 @@ class WorkerQueue(object):
         return self._queue.put(f)
 
 
-def format_storage_config(cls, header=True):
-    if header is True:
-        yield '[storage example_for_{}]'.format(cls.storage_name)
-    yield 'type = {}'.format(cls.storage_name)
-
-    from ..storage.base import Storage
-    from ..utils import get_storage_init_specs
-    handled = set()
-    for spec in get_storage_init_specs(cls, stop_at=Storage):
-        defaults = spec.defaults or ()
-        defaults = dict(zip(spec.args[-len(defaults):], defaults))
-        for key in spec.args[1:]:
-            if key in handled:
-                continue
-            handled.add(key)
-
-            comment = '' if key not in defaults else '#'
-            value = defaults.get(key, '...')
-            yield '{}{} = {}'.format(comment, key, json.dumps(value))
-
-
 def assert_permissions(path, wanted):
     permissions = os.stat(path).st_mode & 0o777
     if permissions > wanted:
         cli_logger.warning('Correcting permissions of {} from {:o} to {:o}'
                            .format(path, permissions, wanted))
         os.chmod(path, wanted)
+
+
+def handle_collection_not_found(config, collection, e=None):
+    storage_name = config.get('instance_name', None)
+
+    cli_logger.warning('{}No collection {} found for storage {}.'
+                       .format('{}\n'.format(e) if e else '',
+                               json.dumps(collection), storage_name))
+
+    if click.confirm('Should vdirsyncer attempt to create it?'):
+        storage_type = config['type']
+        cls, config = storage_class_from_config(config)
+        config['collection'] = collection
+        try:
+            args = cls.create_collection(**config)
+            args['type'] = storage_type
+            return args
+        except NotImplementedError as e:
+            cli_logger.error(e)
+
+    raise exceptions.UserError(
+        'Unable to find or create collection "{collection}" for '
+        'storage "{storage}". Please create the collection '
+        'yourself.'.format(collection=collection,
+                           storage=storage_name))
