@@ -10,10 +10,11 @@ from inspect import getfullargspec
 import requests
 from requests.exceptions import HTTPError
 
-from .base import Item, Storage, normalize_meta_value
-from .http import HTTP_STORAGE_PARAMETERS, USERAGENT, prepare_auth, \
+from .base import Storage, normalize_meta_value
+from .. import exceptions, http, utils
+from ..http import HTTP_STORAGE_PARAMETERS, USERAGENT, prepare_auth, \
     prepare_client_cert, prepare_verify
-from .. import exceptions, utils
+from ..vobject import Item
 
 
 dav_logger = logging.getLogger(__name__)
@@ -40,6 +41,22 @@ def _contains_quoted_reserved_chars(x):
     return False
 
 
+def _assert_multistatus_success(r):
+    # Xandikos returns a multistatus on PUT.
+    try:
+        root = _parse_xml(r.content)
+    except InvalidXMLResponse:
+        return
+    for status in root.findall('.//{DAV:}status'):
+        parts = status.text.strip().split()
+        try:
+            st = int(parts[1])
+        except (ValueError, IndexError):
+            continue
+        if st < 200 or st >= 400:
+            raise HTTPError('Server error: {}'.format(st))
+
+
 def _normalize_href(base, href):
     '''Normalize the href to be a path only relative to hostname and
     schema.'''
@@ -86,7 +103,7 @@ def _parse_xml(content):
 
 def _merge_xml(items):
     if not items:
-        return []
+        return None
     rv = items[0]
     for item in items[1:]:
         rv.extend(item.getiterator())
@@ -133,23 +150,14 @@ class Discover(object):
         _, collection = url.rstrip('/').rsplit('/', 1)
         return urlparse.unquote(collection)
 
-    def find_dav(self):
+    def find_principal(self):
         try:
-            response = self.session.request(
-                'GET', self._well_known_uri, allow_redirects=True,
-                headers=self.session.get_default_headers()
-            )
-            return response.url
+            return self._find_principal_impl('')
         except (HTTPError, exceptions.Error):
-            # The user might not have well-known URLs set up and instead points
-            # vdirsyncer directly to the DAV server.
-            dav_logger.debug('Server does not support well-known URIs.')
-            return ''
-
-    def find_principal(self, url=None):
-        if url is None:
-            url = self.find_dav()
+            dav_logger.debug('Trying out well-known URI')
+            return self._find_principal_impl(self._well_known_uri)
 
+    def _find_principal_impl(self, url):
         headers = self.session.get_default_headers()
         headers['Depth'] = '0'
         body = b"""
@@ -159,8 +167,10 @@ class Discover(object):
             </d:prop>
         </d:propfind>
         """
+
         response = self.session.request('PROPFIND', url, headers=headers,
                                         data=body)
+
         root = _parse_xml(response.content)
         rv = root.find('.//{DAV:}current-user-principal/{DAV:}href')
         if rv is None:
@@ -171,11 +181,10 @@ class Discover(object):
                 'No current-user-principal returned, re-using URL {}'
                 .format(response.url))
             return response.url
-        return urlparse.urljoin(response.url, rv.text)
+        return urlparse.urljoin(response.url, rv.text).rstrip('/') + '/'
 
-    def find_home(self, url=None):
-        if url is None:
-            url = self.find_principal()
+    def find_home(self):
+        url = self.find_principal()
         headers = self.session.get_default_headers()
         headers['Depth'] = '0'
         response = self.session.request('PROPFIND', url,
@@ -187,7 +196,7 @@ class Discover(object):
         rv = root.find('.//' + self._homeset_tag + '/{DAV:}href')
         if rv is None:
             raise InvalidXMLResponse('Couldn\'t find home-set.')
-        return urlparse.urljoin(response.url, rv.text)
+        return urlparse.urljoin(response.url, rv.text).rstrip('/') + '/'
 
     def find_collections(self):
         rv = None
@@ -201,6 +210,23 @@ class Discover(object):
         dav_logger.debug('Given URL is not a homeset URL')
         return self._find_collections_impl(self.find_home())
 
+    def _check_collection_resource_type(self, response):
+        if self._resourcetype is None:
+            return True
+
+        props = _merge_xml(response.findall(
+            '{DAV:}propstat/{DAV:}prop'
+        ))
+        if props is None or not len(props):
+            dav_logger.debug('Skipping, missing <prop>: %s', response)
+            return False
+        if props.find('{DAV:}resourcetype/' + self._resourcetype) \
+           is None:
+            dav_logger.debug('Skipping, not of resource type %s: %s',
+                             self._resourcetype, response)
+            return False
+        return True
+
     def _find_collections_impl(self, url):
         headers = self.session.get_default_headers()
         headers['Depth'] = '1'
@@ -209,8 +235,7 @@ class Discover(object):
         root = _parse_xml(r.content)
         done = set()
         for response in root.findall('{DAV:}response'):
-            props = _merge_xml(response.findall('{DAV:}propstat/{DAV:}prop'))
-            if props.find('{DAV:}resourcetype/' + self._resourcetype) is None:
+            if not self._check_collection_resource_type(response):
                 continue
 
             href = response.find('{DAV:}href')
@@ -346,7 +371,7 @@ class DAVSession(object):
 
         more = dict(self._settings)
         more.update(kwargs)
-        return utils.http.request(method, url, session=self._session, **more)
+        return http.request(method, url, session=self._session, **more)
 
     def get_default_headers(self):
         return {
@@ -490,6 +515,8 @@ class DAVStorage(Storage):
             headers=headers
         )
 
+        _assert_multistatus_success(response)
+
         # The server may not return an etag under certain conditions:
         #
         #   An origin server MUST NOT send a validator header field (Section
@@ -504,7 +531,7 @@ class DAVStorage(Storage):
         #
         # In such cases we return a constant etag. The next synchronization
         # will then detect an etag change and will download the new item.
-        etag = response.headers.get('etag', 'NULL')
+        etag = response.headers.get('etag', None)
         href = self._normalize_href(response.url)
         return href, etag
 
@@ -552,7 +579,7 @@ class DAVStorage(Storage):
                 continue
 
             props = response.findall('{DAV:}propstat/{DAV:}prop')
-            if not props:
+            if props is None or not len(props):
                 dav_logger.warning('Skipping {!r}, properties are missing.'
                                    .format(href))
                 continue
